from bohra.launcher.Utils import CustomFormatter, _check_path, _check_size_file
import pandas as pd
import pathlib
import os
import logging


# Logger
LOGGER =logging.getLogger(__name__) 
LOGGER.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
ch.setFormatter(CustomFormatter())
fh = logging.FileHandler('bohra_run.log')
fh.setLevel(logging.DEBUG)
formatter = logging.Formatter('[%(levelname)s:%(asctime)s] %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p') 
fh.setFormatter(formatter)
LOGGER.addHandler(ch) 
LOGGER.addHandler(fh)






def _setup_assembly_args(kwargs:dict, command:dict, mtb:False) -> dict:

    command['modules'].append('assemble')
    command['params'].append(f"--assembler {kwargs['assembler']}")
    if kwargs["spades_args"] != "":
        command['params'].append(f"--spades_args {kwargs['spades_args']}")
    
    return command



from bohra.launcher.Utils import CustomFormatter

import logging
import subprocess
import pathlib
import os


# Logger
LOGGER =logging.getLogger(__name__) 
LOGGER.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
ch.setFormatter(CustomFormatter())
fh = logging.FileHandler('bohra.log')
fh.setLevel(logging.DEBUG)
formatter = logging.Formatter('[%(levelname)s:%(asctime)s] %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p') 
fh.setFormatter(formatter)
LOGGER.addHandler(ch) 
LOGGER.addHandler(fh)

    
def check_dependencies(check:str = "install",
                       envs:str=f"{pathlib.Path(__file__).parent.parent}/environments",
                       force_reinstall:str="false", tool:str="all")->int:
    """
    Install bohra dependencies.
    """
    script_path = f"{pathlib.Path(__file__).parent}"
    LOGGER.info(f"Will now try {check} dependencies. Please be patient this may take some time!!... Maybe get coffee.")
    process = subprocess.Popen(['bash', f"{script_path}/bohra_install.sh", f"{envs}", f"{check}", f"{force_reinstall}", f"{tool}"], stdout=subprocess.PIPE, encoding='utf-8')
    while process.poll() is None:
        l = process.stdout.readline().strip() # This blocks until it receives a newline.
        LOGGER.info(f"{l}")

    if process.returncode != 0:
        # LOGGER.info(f"{process.stderr.read()}")
        LOGGER.error(f"Error {check}ing dependencies.")
        raise SystemError
        # return 1
    else:
        LOGGER.info("Dependencies installed successfully.")
        LOGGER.info("Bohra is ready to go!")
        return 0

def _check_databases(db_install:bool=False)->int:
    """
    Check that required databases are installed.
    """
    script_path = f"{pathlib.Path(__file__).parent}"
    LOGGER.info(f"Will now check databases required for Bohra. Please be patient this may take some time!!... Maybe get coffee.")
    db_check_cmd = 'get' if db_install else 'check'
    process = subprocess.Popen(['bash', f"{script_path}/bohra_databases.sh", f"{db_check_cmd}"],stderr=subprocess.STDOUT, stdout=subprocess.PIPE, encoding='utf-8')
    while process.poll() is None:
        l = process.stdout.readline().strip() # This blocks until it receives a newline.
        LOGGER.info(f"{l}")

    if process.returncode != 0:
        LOGGER.error(f"Error checking databases: {process.stderr}")
        # raise SystemError
        # return 1
    else:
        LOGGER.info("Databases checked successfully.")
        LOGGER.info("Bohra is ready to go!")
        return 0


import click
import pathlib
import os
from bohra.launcher.TestBohra import run_tests

@click.command()
@click.option('--cpus',
              default=1,
              help="Number of CPUs to use for testing Bohra installation.") 
def test(cpus:int=1):
    """
    Check that bohra is installed correctly and runs as expected.
    """
    
    run_tests(cpus=cpus)


import click
import pathlib
import os

from bohra.launcher.CheckDeps import _check_databases


@click.command()
@click.option('--setup_databases',
              is_flag=True,
            #   default=False,
              help="Download and/or setup required databases.")
def init_databases(setup_databases:bool=False):
    """
    Check that dependencies are installed correctly.
    """
    """
    Install dependencies for Bohra - Highly recommended to run this before running the pipeline.
    """
    print("Will now install dependencies for Bohra.")
    print("This will take some time.")
    print("Please be patient.")
    db_install = True if setup_databases else False
    print(f"Will now check your databases{' and install anything that may be required' if db_install else ''}. ")
    _check_databases(db_install=db_install)
    


from bohra.launcher.Utils import CustomFormatter, _check_path, _run_subprocess, _get_required_columns, _get_columns_list

import pandas as pd
import pathlib
import os
import logging
import shutil

# Logger
LOGGER =logging.getLogger(__name__) 
LOGGER.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
ch.setFormatter(CustomFormatter())
fh = logging.FileHandler('bohra_run.log')
fh.setLevel(logging.DEBUG)
formatter = logging.Formatter('[%(levelname)s:%(asctime)s] %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p') 
fh.setFormatter(formatter)
LOGGER.addHandler(ch) 
LOGGER.addHandler(fh)



def _open_input_file(_file:str) -> pd.DataFrame:
        
    """
    Open input file and return a pandas dataframe
    :param _file: path to input file
    :return: pandas dataframe
    """
    LOGGER.info(f"Opening input file {_file}.")
    if _file == "":
        LOGGER.critical(f"Input file cannot be empty. Please try again.")
        raise SystemExit
    elif not _check_path(_file):
        LOGGER.critical(f"Input file {_file} does not exist or is not accessible.")
        raise SystemExit
    else:
        LOGGER.info(f"Backing up original file {_file} to {pathlib.Path(_file)}.bak.")
        shutil.copy(_file, f"{pathlib.Path(_file)}.bak")
        return pd.read_csv(_file, engine='python', sep = None, dtype = str)

def _check_data_format(df:pd.DataFrame) -> pd.DataFrame:
    """
    Check if the input file is in the correct format
    :param df: pandas dataframe
    :return: pd.DataFrame
    """
    
    columns_req = _get_required_columns()
    columns = _get_columns_list()
    # check if all required columns are present
    for col in columns_req["required"]:
        if col not in df.columns.tolist():
            LOGGER.critical(f"Column {col} is missing from input file.")
            raise SystemExit
    # check if all must have columns are present
    mchk = 0
    for col in columns_req["must_have"]:
        if col in df.columns.tolist():
            mchk = mchk + 1
    if mchk == 0:
        LOGGER.critical(f"Must have at least one of {','.join(columns_req['must_have'])} in your input file.")
        raise SystemExit
    # check if all optional columns are present
    for col in columns:
        if col not in df.columns:
            df[col] = "not_supplied"
    
    for col in df.columns:
        if col not in columns:
            columns.append(col)
    df = df[columns]
    df = df.fillna("not_supplied")
    LOGGER.info(f"Saving input file with columns {','.join(columns)} to {pathlib.Path('input_checked.tsv')}.")
    df.to_csv('input_checked.tsv', sep='\t', index=False, header=True)

    return df

def _check_sequence_file(_file:str) -> bool:
    """ Check if the sequence file exists and is accessible 
    :param _file: path to sequence file
    :return: boolean
    """
        
    if _file.exists() and os.access(_file, os.R_OK):
        
        LOGGER.info(f"Found {_file.name}.")
        return True
    else:
        LOGGER.warning(f"The {_file.name} does not exist or is not accessible.")
        return False

def _get_input_types(cols_provided: list, cols_required:list) -> dict:

    """
    Get the input types from the columns provided
    :param cols_provided: list of columns provided in the input file
    :return: list of input types
    """
    input_types = {}
    for c in cols_required:
        if c == "assembly":
            input_types[c] = "contigs.fa"
        elif c == "r1":
            input_types[c] = "R1.fastq.gz"
        elif c == "r2":
            input_types[c] = "R2.fastq.gz"
    return input_types

def _infer_correct_read(file_path:str) -> str:

    if '_R1' in file_path or '_1.fastq' in file_path or 'r1' in file_path:
        return "R1.fastq.gz"
    elif '_R2' in file_path or '_2.fastq' in file_path or 'r2' in file_path:
        return "R2.fastq.gz"
    else:
        return "unknown"

def _check_outdir(workdir:str, report_outdir:str, replace_report:bool = False) -> bool:

    out = pathlib.Path(workdir,report_outdir)
    if out.exists() and not replace_report:
        LOGGER.critical(f"Output directory {out} already exists. Please choose a different output directory or use the --replace-report flag to overwrite.")
        raise SystemExit
    elif out.exists() and replace_report:
        LOGGER.warning(f"Output directory {out} already exists. Overwriting as per user request.")
        shutil.rmtree(out)
        out.mkdir(parents=True, exist_ok=True)
        return True
    return True

def _make_workdir(_input:pd.DataFrame, workdir:str, report_outdir:str, replace_report:bool = False) -> bool:

    columns = _get_columns_list()
    pereads = ["r1","r2"]
    if _check_path(workdir) and _check_outdir(workdir, report_outdir, replace_report):
        
        wd = pathlib.Path(workdir)
        input_table = _open_input_file(_file=_input)
        input_table = _check_data_format(df=input_table)
        
        input_types = _get_input_types(cols_provided=input_table.columns.tolist(), cols_required=columns)
        for row in input_table.iterrows():
            
            for input_type in input_types:
                user_supplied = row[1][input_type]
                
                if _check_sequence_file(pathlib.Path(user_supplied)):
                    # check that col is actually the r1 or r2
                    target_file = input_types[input_type]
                    if input_type in pereads:
                        target_file = _infer_correct_read(user_supplied)
                    if target_file == "unknown":
                        LOGGER.warning(f"Could not infer read type from {user_supplied}. Skipping link creation.")
                        continue
                    LOGGER.info(f"Linking {user_supplied} to {wd / row[1][columns[0]]}/{target_file}.")
                    try:
                        LOGGER.info(f"Creating directory {row[1][columns[0]]} in {workdir}")
                        target = pathlib.Path(wd / row[1][columns[0]] / target_file)
                        if not target.exists():
                            pathlib.Path(f"{wd / row[1][columns[0]]}").mkdir(parents=True, exist_ok=True)
                            target.symlink_to(user_supplied)
                            if target.exists():
                                LOGGER.info(f"Successfully linked {user_supplied} to {target}.")
                            else:
                                LOGGER.critical(f"{target} does not exist after linking.")
                                raise SystemExit
                        else:
                            LOGGER.warning(f"File {target} already exists. Skipping link creation.")
                    except Exception as e:
                        LOGGER.critical(f"Could not link {user_supplied} to {wd / row[1][columns[0]]}/{target_file}. Error: {e}")
                        raise SystemExit
                else:
                    LOGGER.warning(f"File {user_supplied} does not exist or is not accessible. Skipping link creation.")
    return "input_checked.tsv"



#!/usr/bin/env python3

import pandas as pd
import sys

def make_matrix(df, col1, col2, vals) -> pd.DataFrame:
    """
    Create a matrix from two columns of a DataFrame and a list of values.

    Args:
        df (pd.DataFrame): The input DataFrame.
        col1 (str): The name of the first column.
        col2 (str): The name of the second column.
        vals (list): A list of values to include in the matrix.

    Returns:
        pd.DataFrame: A DataFrame representing the matrix.
    """
    idx = sorted(set(df[col1]).union(df[col2]))
    # Create a pivot table
    matrix = df.pivot(index = col1, columns = col2, values = vals).reindex(index = idx, columns = idx).fillna(0, downcast = 'infer').pipe(lambda x: x+x.values.T)
    # Reindex to include all values
    matrix = matrix.reset_index()
    matrix = matrix.rename(columns = {col1: 'Isolate'})

    return matrix


pth = sys.argv[1]
df = pd.read_csv(pth, sep = '\t')
matrix = make_matrix(df, 'Sample1', 'Sample2', 'Distance')
matrix.to_csv("distances.tsv", sep = '\t', index = False)


#!/usr/bin/env python3
import numpy as np
import pandas as pd
import sys

def get_lower_triangular_matrix(matrix, isos):
    """
    Converts a square matrix to a lower triangular matrix.

    Args:
        matrix: A list of lists representing the square matrix.

    Returns:
        A list of lists representing the lower triangular matrix.
    """
    
    rows = len(matrix)

    lower_triangle = [[len(isos)], [isos[0]]]
    for i in range(1,rows):
    
        r = [isos[i]]
        for j in range(i + 1):
            
            if i != j:
            # print(matrix[i][j])
                r.append(matrix[i][j])
        lower_triangle.append(r)

    
    lines = []
    for row in lower_triangle:
        line = '\t'.join(map(str, row))
        lines.append(line)
    


    return '\n'.join(lines)
        #     lower_triangular[i][j] = matrix[i][j]
    # return lower_triangular

#Example
matrix = pd.read_csv(sys.argv[1], sep='\t')
isos = matrix.iloc[:, 0].values
iso_col = matrix.columns[0]
lower_triangle = get_lower_triangular_matrix(matrix.set_index(iso_col).values.tolist(), isos)
print(lower_triangle)




from bohra.launcher.Utils import CustomFormatter, _check_path, _check_size_file
from bohra.launcher.BohraSetupFiles import _open_input_file, _check_sequence_file
from bohra.launcher.BohraAssembly import  _setup_assembly_args
import pandas as pd
import pathlib
import os
import logging


# Logger
LOGGER =logging.getLogger(__name__) 
LOGGER.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
ch.setFormatter(CustomFormatter())
fh = logging.FileHandler('bohra_run.log')
fh.setLevel(logging.DEBUG)
formatter = logging.Formatter('[%(levelname)s:%(asctime)s] %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p') 
fh.setFormatter(formatter)
LOGGER.addHandler(ch) 
LOGGER.addHandler(fh)


def _check_databases(path :str, dtbtype:str) -> dict:
    
    if path == "":
        LOGGER.info(f"No database path provided using default databases installed for {dtbtype}.")
        
        return f"--{dtbtype} no_db"
    elif _check_path(path):
        return f"--{dtbtype} {path}"


def _missing_reads(input_table: pd.DataFrame) -> bool:

    noctg = input_table[input_table['assembly'] == 'no_contigs']
    if noctg.empty:
        LOGGER.info("There is nothing missing from the input file.")
        res = False
    elif not noctg.empty:
        res = False
        for row in noctg.iterrows():
            if not _check_sequence_file(pathlib.Path(row[1]['r1'])):
                LOGGER.warning(f"Missing reads for {row[1]['Isolate']}.")
                res =  True
        
    return res

def _setup_typing_args(kwargs:dict, command:dict, mtb:False) -> dict:

    typing_dbs = ["blast_db", "data_dir", "mobsuite_db"]
    input_table = _open_input_file(kwargs['input_file'])
    command = _setup_assembly_args(kwargs, command, mtb)
    command['modules'].append('typing')
    for d in typing_dbs:
        db = _check_databases(path = kwargs[d], dtbtype = d)
        command["params"].append(db)

    exclude = eval(kwargs['mlst_exclude'])
    
    if exclude != []:
        command['params'].append(f"--mlst_exclude {','.join(exclude)}")

    return command


import pathlib
import datetime
import pandas as pd
import logging
from bohra.launcher.Utils import CustomFormatter, _check_path
# Logger
# Logger
LOGGER =logging.getLogger(__name__) 
LOGGER.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
ch.setFormatter(CustomFormatter())
fh = logging.FileHandler('bohra.log')
fh.setLevel(logging.DEBUG)
formatter = logging.Formatter('[%(levelname)s:%(asctime)s] %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p') 
fh.setFormatter(formatter)
LOGGER.addHandler(ch) 
LOGGER.addHandler(fh)


CFG = {
    "reads": {
        "ext":"*.f*q.gz",
        "num_expected":2,
    },
    "se-reads": {
        "ext":"*.f*q*",
        "num_expected":1
    },
    "ont": {
        "ext":"*.f*q*",
        "num_expected":1
    },
    "asm": {
        "ext":"*.f*a",
        "num_expected":1
    }
}
  
    
def _get_isolate_list(reads:str,
                      contigs:str,
                      ) -> list:
    """
    Retrieve a list of unique isolate names from files in the specified directory.

    :param _dir: Path to the directory containing the files.
    :param isolates: A list of isolate names to filter the results. If empty, all isolates are returned.
    :param ext: File extension pattern to search for (default is "*.f*q.gz").
    :return: A list of unique isolate names found in the directory.
    """
    # iso_found = _get_isolate_list(_dir=_dir, isolates=isolates, ext="*.f*q.gz")
    all_data = []
    if reads != '' and _check_path(reads):
        ext = CFG['reads']['ext']
        reads_list = sorted(pathlib.Path(reads).rglob(ext))
        all_data.extend(reads_list)
    elif contigs != '' and _check_path(contigs):
        ext = CFG['asm']['ext']
        contigs_list = sorted(pathlib.Path(contigs).rglob(ext))
        all_data.extend(contigs_list)
    else:
        LOGGER.error("No valid reads or contigs path provided. Please check your input.")
        raise SystemExit
    
    iso_found = set()

    for sq in all_data:
        nme = sq.name.split('_')[0]
        iso_found.add(nme)

    return list(iso_found)

def _order_pereads(reads: list) -> list:
    """
    Order paired-end reads by their names.

    :param reads: List of read file paths.
    :return: Ordered list of read file paths.
    """
    
    
    r1 = [r for r in reads if "_R1" in r.name or "_1.f" in r.name or "_r1" in r.name]
    r1 = sorted(r1, key=lambda f: f.stat().st_ctime)[-1]
    r2 = [r for r in reads if "_R2" in r.name or "_2.f" in r.name or "_r2" in r.name]
    r2 = sorted(r2, key=lambda f: f.stat().st_ctime)[-1]
    
    return [r1,r2]

def _glob_sequences(_dir: pathlib.Path, 
                    isolates: list, 
                    sequence_type: str) -> pd.DataFrame:
    """
    Process and validate sequence files for the specified sequence type.

    :param _dir: Path to the directory containing sequence files.
    :param isolates: A list of isolate names to filter the results.
    :param sequence_type: Type of sequence data to process (e.g., 'reads', 'asm').
    :return: True if processing is successful, otherwise raises an error.
    """
    data = []
    if sequence_type in CFG:
        
        for iso in isolates:
            
            seqs = sorted(pathlib.Path(_dir).rglob(f"*{iso}{CFG[sequence_type]['ext']}"))
            LOGGER.info(f"Found {len(seqs)} {sequence_type} for {iso} in {_dir}.")
            if seqs == []:
                seqs = sorted(pathlib.Path(_dir).rglob(f"{iso}/{CFG[sequence_type]['ext']}"))
            if len(seqs) >= CFG[sequence_type]['num_expected']:
                LOGGER.info(f"Now add {sequence_type} for {iso}")
                if sequence_type == "reads":
                    seqs = _order_pereads(seqs)
                    heads = [f"r{i+1}" for i in range(len(seqs))]
                    if len(seqs) > CFG[sequence_type]['num_expected']:
                        LOGGER.warning(f"Expected 2 reads for {iso}, but found {len(seqs)}. Will use the most recent.")
                elif sequence_type == "asm":
                    heads = ["assembly"]
                else:
                    LOGGER.error(f"Unexpected sequence type: {sequence_type}.")
                    raise SystemExit
                line = dict(zip(heads,seqs))
                line['Isolate'] = iso
                data.append(line)
            elif len(seqs) < CFG[sequence_type]['num_expected']:
                LOGGER.warning(f"There do not appear to be {CFG[sequence_type]['num_expected']} {sequence_type} for {iso}. Skipping")
            
        if data != []:
            return pd.DataFrame(data)
        else:
            LOGGER.warning(f"It appears that no {sequence_type} have been found. Please check your input and try again.")
            raise SystemExit
        
    else:
        LOGGER.critical(f"An error has occurred somewhere - {sequence_type} is not a valid type of input data.")
        raise SystemExit

        
def _extract_isolates(isolate_ids:str)-> pd.DataFrame:
    """
    Extract isolates from isolate_ids file
    :input - path to isolate_ids file
    :output - list of isolates
    """
    try:
        isolates_df = pd.read_csv(isolate_ids, sep="\t")
        if 'Isolate' not in isolates_df.columns:
            LOGGER.error("The provided isolate_ids file does not contain a column named 'Isolate'. Please check the file format.")
            raise SystemExit
        if 'species' not in isolates_df.columns:
            LOGGER.warning("The provided isolate_ids file does not contain a column named 'species'. This column is optional but recommended for further annotation of trees.")
        
    except Exception as e:
        LOGGER.error(f"An error occurred while reading the isolate_ids file: {e}")
        raise SystemExit
    
    return isolates_df
    
def find_data(reads:str,
              contigs:str,
              isolate_ids:str,
              outname:str,
              ) -> None:
    """
    Find data in the path provided
    :input - path to reads/contigs
                isolate_ids - file containing isolate ids
                input_type - type of input data (reads, asm)
    :output - list of data
    """
    isolates = []
    bohra_table = []
    if isolate_ids != '' and _check_path(isolate_ids):
        LOGGER.info(f"Extracting isolates from {isolate_ids}")
        isolates_df = _extract_isolates(isolate_ids)
        bohra_table.append(isolates_df)
        isolates = list(isolates_df['Isolate'].unique())
    else:
        LOGGER.warning("No isolate_ids file provided. Using all isolates found in the reads/contigs directories.")
        isolates = _get_isolate_list(reads=reads, contigs = contigs)
    # now get reads
    if reads != '' and _check_path(reads):
        reads_path = pathlib.Path(reads)
        if not reads_path.exists():
            LOGGER.error(f"The specified reads path {reads} does not exist.")
            raise SystemExit
        reads_tab = _glob_sequences(_dir=reads_path, isolates=isolates, sequence_type='reads')
        if not reads_tab.empty:
            bohra_table.append(reads_tab)
    if contigs != '' and _check_path(contigs):
        contigs_path = pathlib.Path(contigs)
        if not contigs_path.exists():
            LOGGER.error(f"The specified contigs path {contigs} does not exist.")
            raise SystemExit
        contigs_tab = _glob_sequences(_dir=contigs_path, isolates=isolates, sequence_type='asm')
        if not contigs_tab.empty:
            bohra_table.append(contigs_tab)
    
    if bohra_table:

        bohra_df = pd.DataFrame()
        for df in bohra_table:
            if bohra_df.empty:
                bohra_df = df
            else:
                bohra_df = pd.merge(bohra_df, df, on='Isolate', how='outer')
        # bohra_df = bohra_df.loc[:,~bohra_df.columns.duplicated()]
        bohra_df = bohra_df.sort_values(by='Isolate')
        bohra_df = bohra_df.fillna('not_supplied')
        bohra_df.to_csv(outname,sep = "\t", index=False)
        LOGGER.info(f"Input files generated successfully and saved to '{outname}'.")


#!/usr/bin/env python3

import sys,pandas as pd

_files = {
    "reads": sys.argv[1],
    "asm": sys.argv[2]
}

reads_result = sys.argv[1]
asm_result = sys.argv[2]

def add_input_name(df, name):
    for col in df.columns:
        if col != "Isolate":
            df = df.rename(columns = {f"{col}": f"{col} ({name})"})
        
    return df

tab = pd.DataFrame()

for _file in _files:
    if _files[_file] != "no_results":
        df = pd.read_csv(_files[_file], sep="\t")
        df = add_input_name(df, _file)
        if tab.empty:
            tab = df
        else:
            tab = pd.merge(tab, df, on="Isolate", how="outer")
tab.to_csv(sys.stdout, sep="\t", index=False)


import click
import pathlib
import os

from bohra.launcher.CheckDeps import check_dependencies, _check_databases


@click.command()

@click.option('--install_dbs',
              is_flag=True,
            #   default=False,
              help="Force reinstallation of all dependencies, even if they are already installed.")
@click.option('--setup_databases',
              is_flag=True,
            #   default=False,
              help="Download and/or setup required databases.")
@click.option('--tool',
              # is_flag=True,
              type=click.Choice(['all', 'torstyverse', 'seqquality', 'relationships','snippy', 'ectyper','mob_suite','panaroo','kleborate','stype','tamr','sonneitype','classify-pangenome'], case_sensitive=False),
              default="all",
              help="Install only a specific set of tools from a single environment. Should really only be used for development and/or testing purposes.")

def check_deps(install_dbs:bool=False, setup_databases:bool=False, tool:str="all"):
    """
    Check dependencies for Bohra - Highly recommended to run this before running the pipeline.
    """
    print("Will now check dependencies for Bohra.")
    print("This will take some time.")
    print("Please be patient.")
    print("Will now check and install anything that may be required. ")
    # print(str(force_reinstall).lower())
    check_dependencies(check = "check", force_reinstall='false', tool=tool )
    _check_databases(db_install=install_dbs or setup_databases)



"""Bohra 
.. moduleauthor:: Kristy Horan <kristyhoran15@gmail.com>

Bohra exinct tree kangaroo that lived on the nullarbor

Bohra is microbial genomics pipeline, designed predominantly for use in public health, but may also be useful in research settings. The pipeline takes as input a tab-delimited file with the isolate IDs followed by the path to READ1 and READ2, a reference for alignment and a unique identifier, where reads are illumina paired end reads (other platforms are not supported).

The pipline is based on nullarbor (https://github.com/tseemann/nullarbor) and is designed to be run in high performance computing environment.

Bohra is modular allowing the user to choose between calling SNPs and generating a phylogenetic tree, perform assemblies and detect AMR, perform typing etc; Or use the full pipeline to call SNPs, generate phylogenies, assemble, type and detect the pan-geneome. The output of Bohra is a html report that can be distributed, with downloadable tables and data.
"""


# import logging
# import argparse
import click
import sys

from bohra.version import version
from click.exceptions import UsageError
from click._compat import get_text_stderr

from bohra.launcher.Utils import _get_cmd_options
from bohra.commands.init_databases import init_databases
from bohra.commands.install import install 
from bohra.commands.generate_input import generate_input
from bohra.commands.test import bohratest
from bohra.commands.check_setup import check_setup
from bohra.launcher.BohraRun import run_bohra


def _show_usage_error(self, file=None):
    if file is None:
        file = get_text_stderr()
    color = None
    if self.ctx is not None:
        color = self.ctx.color
        click.echo(self.ctx.get_help() + '\n', file=file, color=color)
    click.echo('ERROR: %s' % self.format_message(), file=file, color=color)

UsageError.show = _show_usage_error

@click.group(no_args_is_help=True,invoke_without_command=True)
@click.version_option(version, message='%(prog)s version %(version)s')
# @click.pass_context
def cli():
    pass

@cli.group(no_args_is_help=True,invoke_without_command=True)
def run():
    """
    Run the Bohra pipeline.
    """
    pass



def create_subcommand_with_options(name, options_dict):
    f"""Dynamically created a subcommand with options from a list."""

    @run.command(name=name, help = f"Help for the {name} pipeline.")
    def run_subcommand(**kwargs):
        # try:
        if len(sys.argv) > 3:
            run_bohra(pipeline=name, kwargs=kwargs)
        else:
            raise UsageError(f"You did not supply any input. Please supply some input to run the {name} pipeline.")
        # except Exception as e:
        #     raise UsageError(f"An error occurred while running the {name} pipeline: {e}")

    for opt in options_dict:
        
        if 'short_name' in opt:
            click.option(
                f"--{opt['name']}",
                f"{opt['short_name']}",
                type=opt.get('type', None),
                default=opt.get('default', None),
                help=opt.get('help', ''),
                show_default= True,
                is_flag= opt.get('is_flag', False),
            )(run_subcommand) # Apply the decorator to the function
        else:
            click.option(
                f"--{opt['name']}",
                type=opt.get('type', None),
                default=opt.get('default', None),
                help=opt.get('help', ''),
                show_default= True,
                is_flag= opt.get('is_flag', False),
            )(run_subcommand)
    return run_subcommand

cmd_opts = _get_cmd_options()

for opt in cmd_opts:
    create_subcommand_with_options(name = opt, options_dict = cmd_opts[opt])

cli.add_command(init_databases.init_databases)
cli.add_command(install.install_deps)
cli.add_command(check_setup.check_deps)
cli.add_command(generate_input.generate_input)
cli.add_command(bohratest.test)



if __name__ == '__main__':
    cli()

